/**
 * 
 */
package infpp.oceanlife;

import javax.swing.ImageIcon;
import static java.lang.Math.*;
import java.awt.*;
import java.io.Serializable;


/**
 * An abstract class implementing the common properties of the different types
 * of ocean object. In particular, an abstract constructor is used.
 * 
 * @author tdu, akue
 * 
 */
public abstract class OceanObject extends Thread implements Serializable {

	/**
	 * For unique ocean object id's.
	 */
	static int id;

	/**
	 * ID for each ocean object.
	 */
	protected int myID = 0;

	/**
	 * ID for each ocean object.
	 */
	private ImageIcon objectIcon;

	/**
	 * A flag to indicate whether the object is currently highlighted.
	 */
	private boolean isHighlighted = false;

	/**
	 * The currently-used ocean.
	 */
	protected Ocean currentOcean;

	/**
	 * A flag indicating whether the object's thread should die.
	 */
	private boolean toDie = false;

	/**
	 * Used to indicate that no collision has occurred.
	 */
	protected static final int NO_COLLISION = 0;

	/**
	 * Used to signal that the object has collided with the ocean's upper bound.
	 */
	protected static final int COLLISION_UPPER_BOUND = 1;

	/**
	 *Used to signal that the object has collided with the ocean's lower bound.
	 */
	protected static final int COLLISION_LOWER_BOUND = 2;

	/**
	 * Used to signal that the object has collided with the ocean's left bound.
	 */
	protected static final int COLLISION_LEFT_BOUND = 3;

	/**
	 * Used to signal that the object has collided with the ocean's right bound.
	 */
	protected static final int COLLISION_RIGHT_BOUND = 4;

	/**
	 * The object's position vector.
	 */
	private int[] position = new int[2];

	/**
	 * The object's direction vector.
	 */
	protected int[] direction = new int[2];

	/**
	 * @param x
	 *            The x-coordinate of the ocean object.
	 * @param y
	 *            The y-coordinate of the ocean object.
	 * @param objectIcon
	 *            Icons for the ocean objects.
	 * @param currentOcean
	 *            The current used ocean.
	 * @throws Exception
	 *             Throws exception for the position.
	 */
	protected OceanObject(int x, int y, ImageIcon objectIcon, Ocean currentOcean)
			throws Exception {

		this.currentOcean = currentOcean;
		int[] position = new int[2];

		position[0] = x;
		position[1] = y;
		this.setPositionWhenAdding(position);
		this.setObjectIcon(objectIcon);
		this.myID = getUniqueOceanObjectID();

	}

	/**
	 * @return The object icon.
	 */
	public ImageIcon getObjectIcon() {
		return objectIcon;
	}

	/**
	 * @param objectIcon
	 *            the objectIcon to set
	 */
	public void setObjectIcon(ImageIcon objectIcon) {
		this.objectIcon = objectIcon;
	}

	protected void updateDirection() {
		// do nothing
		// is overwritten in the subclasses
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	public void run() {

		while (!toDie) {

			if (currentOcean.isOceanGUIRunning()) {

				Point oldPosition = new Point(getPositionX(), getPositionY());

				int[] newPosition = { oldPosition.x + direction[0],
						oldPosition.y + direction[1] };

				try {
					updateDirection();
					setPositionWhenMoving(newPosition);

				} catch (Exception e) {

					System.out.println("can't move object");
				}
			}
			try {
				Thread.sleep(currentOcean.getAnimationSpeed());
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				System.out.println("Sleeping failed");
			}

		}
	}

	/**
	 * @return An unused ID to use when an object's constructor requests one.
	 */
	public static int getUniqueOceanObjectID() {
		id = id + 1;
		return id;
	}

	/**
	 * @return Returns the id of the ocean object.
	 */
	public int getObjectID() {
		return myID;
	}

	/**
	 * @return Returns the id of the ocean object as type string.
	 */
	public String getObjectIDString() {
		return Integer.toString(myID);
	}

	/**
	 * @return the highlight status of the ocean object as boolean.
	 */
	public boolean isHighlighted() {
		return isHighlighted;
	}

	/**
	 * @param isHighlighted
	 *            whether or not the object should be highlighted.
	 */
	public void setHighlighted(boolean isHighlighted) {
		this.isHighlighted = isHighlighted;
	}

	/**
	 * @return Returns the position vector as an integer array.
	 */
	public int[] getPosition() {
		return position;
	}

	/**
	 * @return the x coordinate of the object's position
	 */
	public int getPositionX() {
		return position[0];
	}

	/**
	 * @return the y coordinate of the object's position
	 */
	public int getPositionY() {
		return position[1];
	}

	/**
	 * @param position
	 *            the position to set
	 */
	public void setPositionWhenAdding(int[] position) throws Exception {
		// diese Methode wird aufgerufen, wenn ein Objekt zum Ozean
		// hinzugefuegt wird. Sie schlaegt jedoch fehl, wenn ein Objekt im
		// Ozean herumschwimmt und an einen der Raender gelangt.
		/**
		 * This method gets called when an object is added to the ocean and sets
		 * its position. It fails (semantically) if used to check the position
		 * of objects already in the ocean.
		 */
		if (position[0] < 0 || position[1] < 0) {

			throw new IllegalArgumentException(
					"Can't add object outside of Ocean");
		} else {
			this.position = position;
		}

	}

	/**
	 * @return an integer specifying which, if any, bounds the object has
	 *         collided with.
	 */
	protected int objectCollidesWithOceanBounds() {
		if (getPositionX() + getDimensionsOfIcon().width > currentOcean
				.getWidth()
				- direction[0]) {
			// das Objekt stoesst am rechten Rand an
			return COLLISION_RIGHT_BOUND;

		}

		if (getPositionX() <= -direction[0]) {
			// das Objekt stoesst am linken Rand an
			return COLLISION_LEFT_BOUND;

		}

		if (getPositionY() + getDimensionsOfIcon().height > currentOcean
				.getDepth()
				- direction[1]) {
			// das Objekt stoesst am unteren Rand an
			return COLLISION_LOWER_BOUND;

		}

		if (getPositionY() <= -direction[1]) {
			// das Objekt stoesst am oberen Rand an
			return COLLISION_UPPER_BOUND;
		}
		return NO_COLLISION; // no collision
	}

	/**
	 * This method is responsible for setting the position of objects already in
	 * the ocean.
	 * 
	 * @param newPosition
	 *            the object's new position
	 * @throws Exception
	 *             if the object is about to swim out of the ocean
	 */
	public void setPositionWhenMoving(int[] newPosition) throws Exception {

		this.position[0] = this.position[0] + direction[0];
		this.position[1] = this.position[1] + direction[1];

	}

	/**
	 * @return a boolean indicating whether the object's thread should die.
	 */
	public boolean isToDie() {
		return toDie;
	}

	/**
	 * @param toDie
	 *            a boolean indicating whether the object's thread should die.
	 */
	public void setToDie(boolean toDie) {
		this.toDie = toDie;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	/**
	 * @return a string representation of the object.
	 */
	@Override
	public String toString() {

		String id = getObjectIDString();
		String upperLeftCornerX = Integer.toString(position[0]);
		String topLeftCornerY = Integer.toString(position[1]);
		String lowerRightCornerX = Integer.toString(position[0]
				+ objectIcon.getIconWidth());
		String lowerRightCornerY = Integer.toString(position[1]
				+ objectIcon.getIconHeight());
		String directionX = Integer.toString(direction[0]);
		String directionY = Integer.toString(direction[1]);

		return "\tid: " + id + "\t\tx: " + upperLeftCornerX + " \ty: "
				+ topLeftCornerY + "\tup to\tx: " + lowerRightCornerX + "\ty: "
				+ lowerRightCornerY + "\tdirection: [" + directionX + ","
				+ directionY + "]" + "\n";
	}

	/**
	 * @return a Dimension containing the width and height of the object's icon.
	 */
	public Dimension getDimensionsOfIcon() {

		Dimension iconDimension = new Dimension(0, 0);

		iconDimension.height = objectIcon.getIconHeight();
		iconDimension.width = objectIcon.getIconWidth();

		return iconDimension;

	}

}
